#include <form.h>
#include <mask.h>
#include <printapp.h>
#include <recarray.h>
#include <tabutil.h>
#include <utility.h>
#include <lffiles.h>

#include "at1.h"

// nomi campi maschera
#include "at1600a.h"
          
// nomi dei campi
#include "soggetti.h"
#include "sezioni.h"

class TSbloccoEsclusi : public TPrintapp
{
	TMask*						_msk;
	TRelation*   			_rel;
	
	int 							_cur;	
	TDate							_data_sblocco; 			// data scadenza esclusione fino a _data_sblocco
	TDate							_data_stampa;
	TString16					_codsez, _codsot;
	bool							_definitiva;

	static bool filter_func_sbloccoesclusi(const TRelation* rel);

protected:
	virtual bool user_create();
	virtual bool user_destroy();
	virtual bool set_print(int m);
	virtual void set_page(int file, int cnt);
	virtual bool preprocess_page(int file, int counter);
	
public:
  void filtra_sezioni();
  void crea_intestazione();
  void header_sezione(const TString16 codsez, const TString16 codsot);
	TSbloccoEsclusi() : _data_sblocco(TODAY), _data_stampa(TODAY) {}
};

HIDDEN inline TSbloccoEsclusi& app() { return (TSbloccoEsclusi&) main_app(); }

void TSbloccoEsclusi::filtra_sezioni()
{
 	const TString16 sezini = _msk->get(F_SEZINI);
 	const TString16 sotini = _msk->get(F_SOTINI);
 	const TString16 sezfin = _msk->get(F_SEZFIN);
 	const TString16 sotfin = _msk->get(F_SOTFIN);
  TRectype da(LF_SOGGETTI);
  TRectype a(LF_SOGGETTI);   
  if (sezini.not_empty())
		da.put(SOG_CODSEZ, sezini);
  if (sotini.not_empty())
		da.put(SOG_CODSOT, sotini);
  if (sezfin.not_empty())
		a.put(SOG_CODSEZ, sezfin);
  if (sotfin.not_empty())
		a.put(SOG_CODSOT, sotfin);
	current_cursor()->setregion(da, a);
}	

bool TSbloccoEsclusi::preprocess_page(int file, int counter)
{ 
	TRectype& recsog = current_cursor()->curr();
	if (_definitiva)
	{
		recsog.put(SOG_ESCLUSO, "  ");
		recsog.put(SOG_TERMESCL, NULLDATE);
		current_cursor()->file().rewrite();
	}	
	// salto pagina se cambio sezione
	const TString16 codsez = recsog.get(SOG_CODSEZ);		
	const TString16 codsot = recsog.get(SOG_CODSOT);		
	if ((_codsez!=codsez)||(_codsot!=codsot))
	{
		if (_codsez != "**")
			printer().formfeed();
		_codsez = codsez;
		_codsot = codsot;
		header_sezione(codsez, codsot);
	}
  return TRUE;
}

void TSbloccoEsclusi::set_page(int file, int cnt)
{ 
	set_row(1,"@0g@pn", FLD(LF_SOGGETTI,SOG_CODICE,"########"));
	set_row(1,"@9g@S", FLD(LF_SOGGETTI,SOG_CATDON));
	set_row(1,"@12g@S", FLD(LF_SOGGETTI,SOG_COGNOME));
	set_row(1,"@38g@S", FLD(LF_SOGGETTI,SOG_NOME));   
	set_row(1,"@63g@ld", FLD(LF_SOGGETTI,SOG_DATANASC));
}

bool TSbloccoEsclusi::filter_func_sbloccoesclusi(const TRelation* rel)
{   
	TRectype& recsog = rel->lfile().curr();
	TDate scadenza = recsog.get(SOG_TERMESCL);
	return (scadenza.ok() && scadenza <= app()._data_sblocco);
}

void TSbloccoEsclusi::header_sezione(const TString16 codsez, const TString16 codsot)
{
	const TString80 densez = current_cursor()->curr(LF_SEZIONI).get(SEZ_DENSEZ);
	const TString80 densot = current_cursor()->curr(LF_SEZIONI).get(SEZ_DENSOT);
	TString intestazione(132);
	intestazione = "Sezione: ";
	intestazione << codsez;
	intestazione << "/";
	intestazione << codsot;
	intestazione << " ";
	intestazione << densez;
	if ((densot.ok())&& (densot.not_empty()))
	{
		intestazione << "/";
		intestazione << densot;
	}		
	intestazione.center_just();
	set_header(1,"@0g%s", (const char*) intestazione);
	return;
}

bool TSbloccoEsclusi::set_print(int)
{
  KEY tasto;
  tasto = _msk->run();
	if (tasto == K_ENTER)
	{  	 
		_codsez = "**";
		_codsot = "**";
		_data_sblocco = _msk->get(F_DATA);
		_definitiva = _msk->get_bool(F_DEFINITIVA);
    reset_files(); 
    add_file(LF_SOGGETTI);
		filtra_sezioni();
    current_cursor()->setfilter("ESCLUSO == \"TD\"", TRUE);
		current_cursor()->set_filterfunction(filter_func_sbloccoesclusi, TRUE);
		reset_print();		
    crea_intestazione();
    return TRUE;
  }
  else
    return FALSE;
}

void TSbloccoEsclusi::crea_intestazione()
{
	reset_header();

	TString sep(132);
	if (_definitiva)
  	sep = "SBLOCCO DEFINITIVO ESCLUSI ";
	else
		sep = "STAMPA PROVVISORIA PER SBLOCCO ESCLUSI ";	  	
  sep << " FINO ALLA DATA ";
	TString16 data_stampa = _data_sblocco.string();
	sep << data_stampa;
	sep.center_just();
 	set_header(2, "@0g%s", (const char*) sep);
 	data_stampa = _data_stampa.string();
  set_header(2,"@0g%10s", (const char*) data_stampa);
  sep = "";
  sep << "Pag. @#";
  set_header(2, "@120g%s", (const char*) sep);  
  	
  sep = "";
  sep.fill('-');
  set_header(3, (const char *) sep);
 	set_header(4,"@0gCod.@9gC.@12gCognome e nome@38gNato il");
 	set_header(5,"@0g--------@9g--@12g-------------------------@38g----------");
}

bool TSbloccoEsclusi::user_create()
{
	_msk = new TMask("at1600a");
	_rel = new TRelation(LF_SOGGETTI);
	_rel->add(LF_SEZIONI,"CODSEZ==CODSEZ|CODSOT==CODSOT");
	//cursore ordinamento per sezione+sottogruppo+cognome+nome	
  _cur = add_cursor(new TCursor(_rel, "", 3));
	return TRUE;
}	

bool TSbloccoEsclusi::user_destroy()	
{
	delete _rel;
	delete _msk;
	return TRUE;
}

int at1600(int argc, char* argv[])
{
	TSbloccoEsclusi a;
	a.run(argc, argv, "Sblocco esclusi");
	return 0;
}