#include <form.h>
#include <mask.h>
#include <printapp.h>
#include <lffiles.h>

#include "soggetti.h"
#include "sezioni.h"
#include <comuni.h>

#include "at6.h"
#include "at6400a.h"

#define ALIAS_LCP	100
#define ALIAS_TCS	200

class TStampaIncompleti : public TPrintapp
{
  static bool filter_func_incompleti(const TRelation* rel);

  TRelation*        _rel;
  TMask*            _msk;
  TAssoc_array			_catdon;
  TAssoc_array			_catnondon;
  int 							_cur;
  TParagraph_string _cognome_nome, _dencom;
  TDate 						_data_stampa;
  TString16					_codsez, _codsot;
  bool							_indirizzo, _datanasc, _categoria, _sezione, _gruppoab0, _rhantid;
  
protected:
  virtual bool user_create();
  virtual bool user_destroy();
  virtual bool set_print(int m);
  virtual void set_page(int file, int cnt);
  virtual bool preprocess_page (int file, int counter);

public:
  void crea_intestazione();
  void filtra_sezioni();
  void header_sezione(const TString16 codsez, const TString16 codsot);
  TMask& app_mask() { return *_msk; }

  TStampaIncompleti() : _data_stampa(TODAY), _cognome_nome("",25), _dencom("",50) {}
};

HIDDEN inline TStampaIncompleti& app() { return (TStampaIncompleti&) main_app(); }

void TStampaIncompleti::filtra_sezioni()
{
 	const TString16 sezini = _msk->get(F_SEZINI);
 	const TString16 sotini = _msk->get(F_SOTINI);
 	const TString16 sezfin = _msk->get(F_SEZFIN);
 	const TString16 sotfin = _msk->get(F_SOTFIN);
  TRectype da(LF_SOGGETTI);
  TRectype a(LF_SOGGETTI);   
	if (sezini.not_empty())  
		da.put(SOG_CODSEZ, sezini);
  if (sotini.not_empty())
		da.put(SOG_CODSOT, sotini);
  if (sezfin.not_empty())
		a.put(SOG_CODSEZ, sezfin);
  if (sotfin.not_empty())
		a.put(SOG_CODSOT, sotfin);
	current_cursor()->setregion(da, a);
}	

void TStampaIncompleti::set_page(int file, int cnt)
{
	set_row(1,"@0g@pn", FLD(LF_SOGGETTI,SOG_CODICE,"########"));
	set_row(1,"@9g@S", FLD(LF_SOGGETTI,SOG_CATDON));
	set_row(1,"@12g#a", &_cognome_nome);
	set_row(1,"@38g@ld", FLD(LF_SOGGETTI,SOG_DATANASC));
	set_row(1,"@49g@S", FLD(LF_SOGGETTI,SOG_DOM_INDIR));
	set_row(1,"@100g@S", FLD(LF_SOGGETTI,SOG_TELABI));
	set_row(1,"@116g@S", FLD(LF_SOGGETTI,SOG_TELLAV));
	set_row(2,"@49g#a", &_dencom);
	set_row(2,"@116g@S", FLD(LF_SOGGETTI,SOG_TELALT));
}

bool TStampaIncompleti::filter_func_incompleti(const TRelation* rel)
{   
	bool filtrato = FALSE;
	bool incompleto = FALSE;
	TLocalisamfile& sog = rel->lfile();
	// filtro per categorie
	TAssoc_array& categorie = app()._catdon;
	if (categorie.items() != 0)
	{
		const TString16 cat = sog.get(SOG_CATDON);
		filtrato = categorie.is_key((const char*) cat);
	}	
	if (!filtrato)
	{	
		// filtro per categorie non donatori
		TAssoc_array& categorie = app()._catnondon;
		if (categorie.items() != 0)
		{
			const TString16 cat1 = sog.get(SOG_CATNOND1);
			const TString16 cat2 = sog.get(SOG_CATNOND2);			
			filtrato = (categorie.is_key((const char*) cat1) || categorie.is_key((const char*) cat2)) ;
		}	
	}		
	if (app()._catdon.items() == 0 && app()._catnondon.items() == 0)
		filtrato = TRUE;
	incompleto = !(app()._indirizzo || app()._datanasc 
							 || app()._categoria || app()._sezione 
							 || app()._gruppoab0 || app()._rhantid);

	if (filtrato && app()._indirizzo && !incompleto)
	{
		TString80 indirizzo = sog.get(SOG_DOM_INDIR);
		TString16 codcom		= sog.get(SOG_DOM_CODCOM);
		incompleto = (indirizzo.empty() || codcom.empty());
	}			
	if (filtrato && app()._datanasc && !incompleto)
	{
		TDate datanasc = sog.get(SOG_DATANASC);
		incompleto = (datanasc.empty());
	}			
	if (filtrato && app()._categoria && !incompleto)
	{
		TString16 catdon = sog.get(SOG_CATDON);
		TString16 catnond1 = sog.get(SOG_CATNOND1);
		TString16 catnond2 = sog.get(SOG_CATNOND2);
		incompleto = (catdon.empty() && catnond1.empty() && catnond2.empty());
	}			
	if (filtrato && app()._sezione && !incompleto)
	{
		TString16 codsez = sog.get(SOG_CODSEZ);
		incompleto = codsez.empty();
	}			
	if (filtrato && app()._gruppoab0 && !incompleto)
	{
		TString16 gruppoab0 = sog.get(SOG_GRUPPOAB0);
		incompleto = gruppoab0.empty();
	}			
	if (filtrato && app()._rhantid && !incompleto)
	{
		TString16 rhantid = sog.get(SOG_RHANTID);
		incompleto = rhantid.empty();
	}			
	filtrato = (filtrato && incompleto);
  return filtrato;
}

void TStampaIncompleti::header_sezione(const TString16 codsez, const TString16 codsot)
{
	const TString80 densez = current_cursor()->curr(LF_SEZIONI).get(SEZ_DENSEZ);
	const TString80 densot = current_cursor()->curr(LF_SEZIONI).get(SEZ_DENSOT);
	TString intestazione(132);
	intestazione = "Sezione: ";
	intestazione << codsez;
	intestazione << "/";
	intestazione << codsot;
	intestazione << " ";
	intestazione << densez;
	if ((densot.ok())&& (densot.not_empty()))
	{
		intestazione << "/";
		intestazione << densot;
	}		
	intestazione.center_just();
	set_header(1,"@0g%s", (const char*) intestazione);
	return;
}

bool TStampaIncompleti::preprocess_page(int file, int counter)
{
	TRectype& recsog = current_cursor()->curr();
	TString80 nome = recsog.get(SOG_COGNOME);
	nome << " ";
	nome << recsog.get(SOG_NOME);
	_cognome_nome = nome;
	TString256 localita = "";
	localita << current_cursor()->curr(-ALIAS_LCP).get("S6");
	if (localita.not_empty() && localita.ok())
	{
		localita << " ";
		localita << current_cursor()->curr(-ALIAS_LCP).get("S0");
		localita << " - ";		
	}
	else
	{
		localita = current_cursor()->curr(LF_COMUNI).get(COM_CAPCOM);
		localita << " ";
	}			
	localita << current_cursor()->curr(LF_COMUNI).get(COM_DENCOM);
	localita.trim();
	_dencom = localita;
	// salto pagina se cambio sezione
	const TString16 codsez = recsog.get(SOG_CODSEZ);		
	const TString16 codsot = recsog.get(SOG_CODSOT);		
	if ((_codsez!=codsez)||(_codsot!=codsot))
	{
		if (_codsez != "**")
			printer().formfeed();
		_codsez = codsez;
		_codsot = codsot;
		header_sezione(codsez, codsot);
	}
  return TRUE;
}

bool TStampaIncompleti::set_print(int m)
{
  KEY tasto;
  tasto = _msk->run();
  if (tasto == K_ENTER)
  {
  	_codsez = "**";
  	_codsot = "**";
    reset_files(); 
    add_file(LF_SOGGETTI);
		filtra_sezioni();
	  const TString16 catpri = _msk->get(F_CAT1);
	  const TString16 catsec = _msk->get(F_CAT2);
	  const TString16 catter = _msk->get(F_CAT3);
	  const TString16 catqua = _msk->get(F_CAT4);
	  const TString16 catqui = _msk->get(F_CAT5);
	  const TString16 catses = _msk->get(F_CAT6);
	  if (catpri.not_empty() && catpri.ok())
	  	_catdon.add((const char*) catpri);
	  if (catsec.not_empty() && catsec.ok())
	  	_catdon.add((const char*) catsec);
	  if (catter.not_empty() && catter.ok())
	  	_catdon.add((const char*) catter);
	  if (catqua.not_empty() && catqua.ok())
	  	_catdon.add((const char*) catqua);
	  if (catqui.not_empty() && catqui.ok())
	  	_catdon.add((const char*) catqui);
	  if (catses.not_empty() && catses.ok())
	  	_catdon.add((const char*) catses);
	  const TString16 catnpri = _msk->get(F_CATN1);
	  const TString16 catnsec = _msk->get(F_CATN2);
	  const TString16 catnter = _msk->get(F_CATN3);
	  const TString16 catnqua = _msk->get(F_CATN4);
	  const TString16 catnqui = _msk->get(F_CATN5);
	  const TString16 catnses = _msk->get(F_CATN6);
	  if (catnpri.not_empty() && catnpri.ok())
	  	_catnondon.add((const char*) catnpri);
	  if (catnsec.not_empty() && catnsec.ok())
	  	_catnondon.add((const char*) catnsec);
	  if (catnter.not_empty() && catnter.ok())
	  	_catnondon.add((const char*) catnter);
	  if (catnqua.not_empty() && catnqua.ok())
	  	_catnondon.add((const char*) catnqua);
	  if (catnqui.not_empty() && catnqui.ok())
	  	_catnondon.add((const char*) catnqui);
	  if (catnses.not_empty() && catnses.ok())
	  	_catnondon.add((const char*) catnses);
		_indirizzo 	= _msk->get_bool(F_INDIRIZZO);	  	
		_datanasc 	= _msk->get_bool(F_DATANASC);	  	
		_categoria	= _msk->get_bool(F_CATEGORIA);	  	
		_sezione 		= _msk->get_bool(F_SEZIONE);	  	
		_gruppoab0 	= _msk->get_bool(F_GRUPPOAB0);	  	
		_rhantid 		= _msk->get_bool(F_RHANTID);	  	
   	//current_cursor()->setfilter("DOM_CODCOM == \"\"");
		current_cursor()->set_filterfunction (filter_func_incompleti);
		reset_print();
    crea_intestazione();
    return TRUE;
  }
  else
    return FALSE;
}

void TStampaIncompleti::crea_intestazione()
{
  reset_header();
	TString sep(132);
	sep = "STAMPA DI CONTROLLO SOGGETTI INCOMPLETI";
	sep.center_just();
	set_header(2, "@0g%s", (const char*) sep);
	TString16 data_stampa = _data_stampa.string();
	set_header(2,"@0g%10s", (const char*) data_stampa);
	sep = "";
	sep << "Pag. @#";
	set_header(2, "@120g%s", (const char*) sep);  
	sep = "";
	sep.fill('-');
	set_header(3, (const char *) sep);
	set_header(4,"@0gCodice@9gC.@12gCognome e nome@38gNato il@49gIndirizzo@100gTelefono abit.@116gTelefono lavoro");
	set_header(5,"@49gCAP/Localit/Comune/Prov.@116gTelefono altro");
	set_header(6,"@0g--------@9g--@12g-------------------------@38g----------@49g--------------------------------------------------@100g---------------@116g---------------");
}

bool TStampaIncompleti::user_create()
{
  _rel = new TRelation(LF_SOGGETTI);
  _rel->add("LCP", "CODTAB==DOM_CODLOC",1,0,ALIAS_LCP);
  _rel->add(LF_COMUNI, "COM==DOM_CODCOM");
  // per stampare nell'intestazione la denominazione della sezione
  _rel->add(LF_SEZIONI, "CODSEZ==CODSEZ|CODSOT==CODSOT");
	// ordinamento per sezione+sottogruppo+cognome+nome  
  _cur = add_cursor(new TCursor(_rel, "", 3));
  _msk = new TMask("at6400a");
  return TRUE;
}

bool TStampaIncompleti::user_destroy()
{
  delete _msk;
  delete _rel;
  return TRUE;
}

int at6400(int argc, char* argv[])
{
  TStampaIncompleti a;
  a.run(argc, argv, "Stampa di controllo soggetti incompleti");
  return 0;
}
